#!/bin/bash
set -euo pipefail

apt-get update
apt-get install -y curl

curl -LsSf https://aider.chat/install.sh | sh
if [ -f "$HOME/.local/bin/env" ]; then
    source "$HOME/.local/bin/env"
fi
aider --version
