from abc import ABC, abstractmethod
from pathlib import Path


class BaseStorage(ABC):
    @abstractmethod
    async def upload_file(self, file_path: Path, remote_path: str) -> None:
        pass

    @abstractmethod
    async def download_file(self, remote_path: str, file_path: Path) -> None:
        pass
