from pathlib import Path

SUPABASE_URL = "https://ofhuhcpkvzjlejydnvyd.supabase.co"
SUPABASE_PUBLISHABLE_KEY = "sb_publishable_Z-vuQbpvpG-PStjbh4yE0Q_e-d3MTIH"

CREDENTIALS_DIR = Path("~/.harbor").expanduser()
CREDENTIALS_PATH = CREDENTIALS_DIR / "credentials.json"
CALLBACK_PORT = 19284
