from pathlib import Path

CACHE_DIR = Path("~/.cache/harbor").expanduser()
TASK_CACHE_DIR = CACHE_DIR / "tasks"
PACKAGE_CACHE_DIR = CACHE_DIR / "tasks" / "packages"
DATASET_CACHE_DIR = CACHE_DIR / "datasets"
NOTIFICATIONS_PATH = CACHE_DIR / "notifications.json"
DEFAULT_REGISTRY_URL = (
    "https://raw.githubusercontent.com/laude-institute/harbor/main/registry.json"
)
HARBOR_REGISTRY_WEBSITE_URL = "https://harbor-registry-website.vercel.app"
HARBOR_REGISTRY_TASKS_URL = f"{HARBOR_REGISTRY_WEBSITE_URL}/tasks"
HARBOR_REGISTRY_DATASETS_URL = f"{HARBOR_REGISTRY_WEBSITE_URL}/datasets"
